@@ -2,18 +2,18 @@
 **  Copyright (c) 2005-2008 Sendmail, Inc. and its suppliers.
 **	All rights reserved.
 **
-**  $Id: dkim-filter.h,v 1.166 2008/03/28 20:48:41 msk Exp $
+**  $Id: dkim-filter.h,v 1.169 2008/04/14 18:03:05 msk Exp $
 */
 
 #ifndef _DKIM_FILTER_H_
 #define _DKIM_FILTER_H_
 
 #ifndef lint
-static char dkim_filter_h_id[] = "@(#)$Id: dkim-filter.h,v 1.166 2008/03/28 20:48:41 msk Exp $";
+static char dkim_filter_h_id[] = "@(#)$Id: dkim-filter.h,v 1.169 2008/04/14 18:03:05 msk Exp $";
 #endif /* !lint */
 
 #define	DKIMF_PRODUCT	"Sendmail DKIM Filter"
-#define	DKIMF_VERSION	"2.5.2"
+#define	DKIMF_VERSION	"2.5.3"
 
 /* libsm */
 #include <sm/gen.h>
@@ -36,7 +36,7 @@ static char dkim_filter_h_id[] = "@(#)$Id: dkim-filter.h,v 1.166 2008/03/28 20:4
 #define	BUFRSZ		1024
 #define	CACHESTATSINT	300
 #define	CBINTERVAL	3
-#define CMDLINEOPTS	"a:Ab:c:C:d:DfF:hi:I:k:KlL:m:M:o:p:P:qrRs:S:t:T:u:U:vVx:?"
+#define CMDLINEOPTS	"a:Ab:c:C:d:DfF:hi:I:k:KlL:m:M:o:p:P:qrRs:S:t:T:u:U:vVWx:?"
 #define	DEFMAXHDRSZ	65536
 #define	DEFTIMEOUT	5
 #define	HOSTUNKNOWN	"unknown-host"
