@@ -1,12 +1,12 @@
 /*
-**  Copyright (c) 2007 Sendmail, Inc. and its suppliers.
+**  Copyright (c) 2007, 2008 Sendmail, Inc. and its suppliers.
 **	All rights reserved.
 **
-**  $Id: dkim-stats.c,v 1.11 2007/06/27 19:49:23 msk Exp $
+**  $Id: dkim-stats.c,v 1.15 2008/02/28 00:05:19 msk Exp $
 */
 
 #ifndef lint
-static char dkim_stats_c_id[] = "@(#)$Id: dkim-stats.c,v 1.11 2007/06/27 19:49:23 msk Exp $";
+static char dkim_stats_c_id[] = "@(#)$Id: dkim-stats.c,v 1.15 2008/02/28 00:05:19 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -20,29 +20,18 @@ static char dkim_stats_c_id[] = "@(#)$Id: dkim-stats.c,v 1.11 2007/06/27 19:49:2
 #include <fcntl.h>
 #include <time.h>
 
-/* libdb includes */
-#include <db.h>
-#ifndef DB_NOTFOUND
-# define DB_NOTFOUND	1
-#endif /* ! DB_NOTFOUND */
-#ifndef DB_VERSION_MAJOR
-# define DB_VERSION_MAJOR	1
-#endif /* ! DB_VERSION_MAJOR */
-
-#define	DB_VERSION_CHECK(x,y,z)	((DB_VERSION_MAJOR == (x) && \
-				  DB_VERSION_MINOR >= (y)) || \
-				 DB_VERSION_MAJOR > (x))
-
 /* sendmail includes */
 #include <sm/gen.h>
 #include <sm/cdefs.h>
 #include <sm/string.h>
 
 /* dkim-filter includes */
+#include "dkim-db.h"
 #include "stats.h"
 
 /* globals */
 char *progname;
+bool dolog = FALSE;			/* XXX -- dkim-db shouldn't use this */
 
 /*
 **  DKIMS_DUMP -- dump a database's contents
@@ -54,7 +43,7 @@ char *progname;
 **  	None.
 */
 
-void
+static void
 dkims_dump(char *path)
 {
 #if ! DB_VERSION_CHECK(2,0,0)
@@ -73,34 +62,13 @@ dkims_dump(char *path)
 	assert(path != NULL);
 
 	/* open the DB */
-#if DB_VERSION_CHECK(3,0,0)
-	status = db_create(&db, NULL, 0);
-	if (status == 0)
-	{
-# if DB_VERSION_CHECK(4,0,0)
-		status = db->open(db, NULL, path, NULL, DB_HASH,
-		                  DB_RDONLY, 0);
-# else /* DB_VERSION_CHECK(4,0,0) */
-		status = db->open(db, path, NULL, DB_HASH, DB_RDONLY, 0);
-# endif /* DB_VERSION_CHECK(4,0,0) */
-	}
-#elif DB_VERSION_CHECK(2,0,0)
-	status = db_open(path, DB_HASH, DB_RDONLY, 0, NULL, NULL, &db);
-#else /* DB_VERSION_CHECK(2,0,0) */
-	db = dbopen(path, O_RDONLY, 0, DB_HASH, NULL);
-	if (db == NULL)
-		status = errno;
-#endif /* DB_VERSION_CHECK */
+	status = dkimf_db_open_ro(&db, path);
 
 	if (status != 0)
 	{
 		char *err;
 
-#if DB_VERSION_CHECK(3,0,0)
-		err = db_strerror(status);
-#else /* DB_VERSION_CHECK(3,0,0) */
-		err = strerror(errno);
-#endif /* DB_VERSION_CHECK(3,0,0) */
+		err = DB_STRERROR(status);
 		fprintf(stderr, "%s: %s: db->open(): %s\n", progname, path,
 		        err);
 
@@ -114,14 +82,10 @@ dkims_dump(char *path)
 	{
 		char *err;
 
-# if DB_VERSION_CHECK(3,0,0)
-		err = db_strerror(status);
-# else /* DB_VERSION_CHECK(3,0,0) */
-		err = strerror(errno);
-# endif /* DB_VERSION_CHECK(3,0,0) */
+		err = DB_STRERROR(status);
 		fprintf(stderr, "%s: %s: db->cursor(): %s\n", progname, path,
 		        err);
-		(void) db->close(db, 0);
+		DKIMF_DBCLOSE(db);
 		return;
 	}
 #endif /* DB_VERSION_CHECK(3,0,0) */
@@ -157,15 +121,11 @@ dkims_dump(char *path)
 		{
 			char *err;
 
-# if DB_VERSION_CHECK(3,0,0)
-			err = db_strerror(status);
-# else /* DB_VERSION_CHECK(3,0,0) */
-			err = strerror(errno);
-# endif /* DB_VERSION_CHECK(3,0,0) */
+			err = DB_STRERROR(status);
 			fprintf(stderr, "%s: %s: dbc->c_get(): %s\n",
 			        progname, path, err);
 			dbc->c_close(dbc);
-			(void) db->close(db, 0);
+			DKIMF_DBCLOSE(db);
 			return;
 		}
 #else /* DB_VERSION_CHECK(2,0,0) */
@@ -178,7 +138,7 @@ dkims_dump(char *path)
 		{
 			fprintf(stderr, "%s: %s: db->seq(): %s\n",
 			        progname, path, strerror(errno));
-			(void) db->close(db);
+			DKIMF_DBCLOSE(db);
 			return;
 		}
 
@@ -203,10 +163,8 @@ dkims_dump(char *path)
 	/* close database */
 #if DB_VERSION_CHECK(2,0,0)
 	(void) dbc->c_close(dbc);
-	(void) db->close(db, 0);
-#else /* DB_VERSION_CHECK(2,0,0) */
-	(void) db->close(db);
 #endif /* DB_VERSION_CHECK(2,0,0) */
+	DKIMF_DBCLOSE(db);
 }
 
 /*
@@ -219,7 +177,7 @@ dkims_dump(char *path)
 **  	EX_USAGE
 */
 
-int
+static int
 usage(void)
 {
 	fprintf(stderr, "%s: usage(): %s path\n", progname, progname);
