@@ -1,12 +1,12 @@
 /*
-**  Copyright (c) 2005-2007 Sendmail, Inc. and its suppliers.
+**  Copyright (c) 2005-2008 Sendmail, Inc. and its suppliers.
 **	All rights reserved.
 **
-**  $Id: util.c,v 1.62 2007/10/31 18:34:11 msk Exp $
+**  $Id: util.c,v 1.69 2008/02/18 02:53:28 msk Exp $
 */
 
 #ifndef lint
-static char util_c_id[] = "@(#)$Id: util.c,v 1.62 2007/10/31 18:34:11 msk Exp $";
+static char util_c_id[] = "@(#)$Id: util.c,v 1.69 2008/02/18 02:53:28 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -33,21 +33,22 @@ static char util_c_id[] = "@(#)$Id: util.c,v 1.62 2007/10/31 18:34:11 msk Exp $"
 # include <regex.h>
 #endif /* _FFR_REPLACE_RULES */
 
+#ifdef SOLARIS
+# if SOLARIS <= 20600
+#  define socklen_t size_t
+# endif /* SOLARIS <= 20600 */
+#endif /* SOLARIS */
+
 /* libsm includes */
 #include <sm/gen.h>
 #include <sm/string.h>
 
-#if POPAUTH
-/* system includes */
-# include <pthread.h>
-
-/* libdb includes */
-# include <db.h>
-#endif /* POPAUTH */
-
 /* dkim-filter includes */
 #include "dkim-filter.h"
 #include "util.h"
+#if POPAUTH || _FFR_BODYLENGTH_DB
+# include "dkim-db.h"
+#endif /* POPAUTH || _FFR_BODYLENGTH_DB */
 
 /* missing definitions */
 #ifndef INADDR_NONE
@@ -81,6 +82,14 @@ static char *optlist[] =
 	"_FFR_ANTICIPATE_SENDMAIL_MUNGE",
 #endif /* _FFR_ANTICIPATE_SENDMAIL_MUNGE */
 
+#if _FFR_AUTHSERV_JOBID
+	"_FFR_AUTHSERV_JOBID",
+#endif /* _FFR_AUTHSERV_JOBID */
+
+#ifdef _FFR_BODYLENGTH_DB
+	"_FFR_BODYLENGTH_DB",
+#endif /* _FFR_BODYLENGTH_DB */
+
 #if _FFR_CAPTURE_UNKNOWN_ERRORS
 	"_FFR_CAPTURE_UNKNOWN_ERRORS",
 #endif /* _FFR_CAPTURE_UNKNOWN_ERRORS */
@@ -225,6 +234,40 @@ dkimf_setmaxfd(void)
 	}
 }
 
+/*
+**  DKIMF_STRIPBRACKETS -- remove angle brackets from the sender address
+**
+**  Parameters:
+** 	addr -- address to be processed
+**
+**  Return value:
+**  	None.
+*/
+
+void
+dkimf_stripbrackets(char *addr)
+{
+	char *p, *q;
+
+	assert(addr != NULL);
+
+	p = addr;
+	q = addr + strlen(addr) - 1;
+
+	while (*p == '<' && *q == '>')
+	{
+		p++;
+		*q-- = '\0';
+	}
+
+	if (p != addr)
+	{
+		for (q = addr; *p != '\0'; p++, q++)
+			*q = *p;
+		*q = '\0';
+	}
+}
+
 /*
 **  DKIMF_LOWERCASE -- lowercase-ize a string
 **
@@ -276,6 +319,7 @@ dkimf_list_lookup(Peer list, char *data)
 	return FALSE;
 }
 
+
 /*
 **  DKIMF_CHECKHOST -- check the peerlist for a host and its wildcards
 **
@@ -473,7 +517,7 @@ dkimf_initpopauth(void)
 
 /*
 **  DKIMF_CHECKPOPAUTH -- check a POP before SMTP database for client
-**                      authentication
+**                        authentication
 **
 **  Parameters:
 **  	db -- DB handle to use for searching
@@ -483,24 +527,16 @@ dkimf_initpopauth(void)
 **  	TRUE iff the database could be opened and the client was verified.
 **
 **  Notes:
-**  	- should be a shared handle rather than a per-thread handle to
-**  	  reduce descriptor consumption
-**  	- needs locking
-**  	- should be able to return TRUE/FALSE as well as errors
-**  	- path should be runtime-configurable rather than hard-coded
 **  	- does the key contain anything meaningful, like an expiry time?
 */
 
 bool
 dkimf_checkpopauth(DB *db, struct sockaddr *ip)
 {
-	bool ret = FALSE;
+	bool exists;
 	int status;
-	int fd;
 	struct sockaddr_in *sin;
 	struct in_addr addr;
-	DBT d;
-	DBT q;
 	char ipbuf[DKIM_MAXHOSTNAMELEN + 1];
 
 	assert(ip != NULL);
@@ -514,108 +550,13 @@ dkimf_checkpopauth(DB *db, struct sockaddr *ip)
 	else
 		sin = (struct sockaddr_in *) ip;
 
-	memset(&d, 0, sizeof d);
-	memset(&q, 0, sizeof q);
 
 	memcpy(&addr.s_addr, &sin->sin_addr, sizeof addr.s_addr);
 
 	dkimf_inet_ntoa(addr, ipbuf, sizeof ipbuf);
-
-	q.data = ipbuf;
-	q.size = strlen(q.data);
-
-	/* we don't care about the value for now */
-# if DB_VERSION_MAJOR > 2
-	d.flags = DB_DBT_USERMEM|DB_DBT_PARTIAL;
-# endif /* DB_VERSION_MAJOR > 2 */
-
-	/* establish read-lock */
-	fd = -1;
-# if DB_VERSION_MAJOR >= 2
-	status = -1;
-	status = db->fd(db, &fd);
-# else /* DB_VERSION_MAJOR >= 2 */
-	status = 0;
-	fd = db->fd(db);
-# endif /* DB_VERSION_MAJOR >= 2 */
-
-	/* XXX -- allow multiple readers? */
-	(void) pthread_mutex_lock(&pop_lock);
-
-	if (status == 0 && fd != -1)
-	{
-# ifdef LOCK_SH
-		status = flock(fd, LOCK_SH);
-
-		if (status != 0 && dolog)
-		{
-			syslog(LOG_WARNING, "flock(LOCK_SH): %s",
-			       strerror(errno));
-		}
-# else /* LOCK_SH */
-		struct flock l;
-
-		l.l_start = 0;
-		l.l_len = 0;
-		l.l_type = F_RDLCK;
-		l.l_whence = SEEK_SET;
-
-		status = fcntl(fd, F_SETLKW, &l);
-
-		if (status != 0 && dolog)
-		{
-			syslog(LOG_WARNING, "fcntl(F_RDLCK): %s",
-			       strerror(errno));
-		}
-# endif /* LOCK_SH */
-	}
-
-# if DB_VERSION_MAJOR < 2
-	status = db->get(db, &q, &d, 0);
-# else /* DB_VERSION_MAJOR < 2 */
-	status = db->get(db, NULL, &q, &d, 0);
-# endif /* DB_VERSION_MAJOR */
-	if (status == 0)
-	{
-		ret = TRUE;
-	}
-	else if (status != DB_NOTFOUND && dolog)
-	{
-		syslog(LOG_ERR, "db->get(): %s", db_strerror(status));
-	}
-
-	/* surrender read-lock */
-	if (fd != -1)
-	{
-# ifdef LOCK_SH
-		status = flock(fd, LOCK_UN);
-
-		if (status != 0 && dolog)
-		{
-			syslog(LOG_WARNING, "flock(LOCK_UN): %s",
-			       strerror(errno));
-		}
-# else /* LOCK_SH */
-		struct flock l;
-
-		l.l_start = 0;
-		l.l_len = 0;
-		l.l_type = F_UNLCK;
-		l.l_whence = SEEK_SET;
-
-		status = fcntl(fd, F_SETLKW, &l);
-
-		if (status != 0 && dolog)
-		{
-			syslog(LOG_WARNING, "fcntl(F_UNLCK): %s",
-			       strerror(errno));
-		}
-# endif /* LOCK_SH */
-	}
-
-	(void) pthread_mutex_unlock(&pop_lock);
-
-	return ret;
+	exists = FALSE;
+	status = dkimf_db_exists(db, ipbuf, &exists, &pop_lock);
+	return (status == 0 && exists);
 }
 #endif /* POPAUTH */
 
@@ -628,10 +569,7 @@ dkimf_checkpopauth(DB *db, struct sockaddr *ip)
 **  	str -- entry
 **
 **  Return value:
-**  	TRUE if successful, FALSE otherwise
-**
-**  Side effects:
-**  	Prints an error message when appropriate.
+**  	TRUE if successful, FALSE otherwise (memory allocation error)
 */
 
 static bool
@@ -645,11 +583,7 @@ dkimf_enqueue(struct Peer **head, struct Peer **tail, char *str)
 
 	newpeer = (struct Peer *) malloc(sizeof(struct Peer));
 	if (newpeer == NULL)
-	{
-		fprintf(stderr, "%s: malloc(): %s\n", progname,
-		        strerror(errno));
 		return FALSE;
-	}
 
 	newpeer->peer_next = NULL;
 
@@ -676,11 +610,7 @@ dkimf_enqueue(struct Peer **head, struct Peer **tail, char *str)
 	}
 
 	if (newpeer->peer_info == NULL)
-	{
-		fprintf(stderr, "%s: strdup(): %s\n", progname,
-		        strerror(errno));
 		return FALSE;
-	}
 
 	return TRUE;
 }
@@ -695,9 +625,6 @@ dkimf_enqueue(struct Peer **head, struct Peer **tail, char *str)
 **
 **  Return value:
 **  	TRUE if successful, FALSE otherwise
-**
-**  Side effects:
-**  	Prints an error message when appropriate.
 */
 
 bool
@@ -747,6 +674,32 @@ dkimf_load_list(FILE *in, char **deflist, struct Peer **list)
 	return TRUE;
 }
 
+/*
+**  DKIMF_FREE_LIST -- destroy a list of peer information
+**
+**  Parameters:
+**  	list -- list to destroy
+**
+**  Return value:
+**  	None.
+*/
+
+void
+dkimf_free_list(struct Peer *list)
+{
+	struct Peer *cur;
+	struct Peer *next;
+
+	assert(list != NULL);
+
+	for (cur = list; cur != NULL; cur = next)
+	{
+		next = cur->peer_next;
+		free(cur->peer_info);
+		free(cur);
+	}
+}
+
 #ifdef _FFR_REPLACE_RULES
 /*
 **  DKIMF_LOAD_REPLIST -- load a list of replace patterns
@@ -828,6 +781,34 @@ dkimf_load_replist(FILE *in, struct replace **list)
 
 	return TRUE;
 }
+
+/*
+**  DKIMF_FREE_REPLIST -- destroy a list of replacement information
+**
+**  Parameters:
+**  	list -- list to destroy
+**
+**  Return value:
+**  	None.
+*/
+
+void
+dkimf_free_replist(struct replace *list)
+{
+	struct replace *cur;
+	struct replace *next;
+
+	assert(list != NULL);
+
+	for (cur = list; cur != NULL; cur = next)
+	{
+		next = cur->repl_next;
+		regfree(&cur->repl_re);
+		free(cur->repl_txt);
+		free(cur);
+	}
+}
+
 #endif /* _FFR_REPLACE_RULES */
 
 /*
