@@ -1,12 +1,12 @@
 /*
-**  Copyright (c) 2007 Sendmail, Inc. and its suppliers.
+**  Copyright (c) 2007, 2008 Sendmail, Inc. and its suppliers.
 **    All rights reserved.
 */
 
 #ifdef QUERY_CACHE
 
 #ifndef lint
-static char dkim_cache_c_id[] = "@(#)$Id: dkim-cache.c,v 1.20 2007/10/24 06:32:56 msk Exp $";
+static char dkim_cache_c_id[] = "@(#)$Id: dkim-cache.c,v 1.25 2008/03/17 16:40:21 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -41,8 +41,11 @@ static char dkim_cache_c_id[] = "@(#)$Id: dkim-cache.c,v 1.20 2007/10/24 06:32:5
 # define DB_VERSION_MAJOR	1
 #endif /* ! DB_VERSION_MAJOR */
 
-#define	DB_VERSION_CHECK(x,y,z)	((DB_VERSION_MAJOR == (x) && \
-				  DB_VERSION_MINOR >= (y)) || \
+#define DB_VERSION_CHECK(x,y,z) ((DB_VERSION_MAJOR == (x) && \
+				  DB_VERSION_MINOR == (y) && \
+				  DB_VERSION_PATCH >= (z)) || \
+				 (DB_VERSION_MAJOR == (x) && \
+				  DB_VERSION_MINOR > (y)) || \
 				 DB_VERSION_MAJOR > (x))
 
 /* data types */
@@ -58,147 +61,7 @@ static pthread_mutex_t cache_stats_lock; /* stats lock */
 static u_int c_hits = 0;		/* cache hits */
 static u_int c_queries = 0;		/* cache queries */
 static u_int c_expired = 0;		/* expired cache hits */
-#if DB_VERSION_MAJOR == 1
 static pthread_mutex_t cache_lock;	/* cache lock */
-#else /* DB_VERSION_MAJOR == 1 */
-# ifdef PTHREAD_RWLOCK_INITIALIZER
-static pthread_rwlock_t cache_rwlock;	/* cache lock */
-# else /* PTHREAD_RWLOCK_INITIALIZER */
-static pthread_mutex_t cache_lock;	/* cache lock */
-static u_int r_wait;			/* readers waiting */
-static u_int r_active;			/* readers active */
-static u_int w_wait;			/* writers waiting */
-static u_int w_active;			/* writers waiting */
-static pthread_cond_t writer_go;	/* writer should enter */
-static pthread_cond_t reader_go;	/* reader should enter */
-# endif /* PTHREAD_RWLOCK_INITIALIZER */
-#endif /* DB_VERSION_MAJOR == 1 */
-
-/*
-**  DKIM_CACHE_READ_LOCK -- acquire a read lock
-**
-**  Parameters:
-**  	None.
-**
-**  Return value:
-**  	None.
-*/
-
-static void
-dkim_cache_read_lock(void)
-{
-#if DB_VERSION_CHECK(2,0,0)
-# ifdef PTHREAD_RWLOCK_INITIALIZER
-	(void) pthread_rwlock_rdlock(&cache_rwlock);
-# else /* PTHREAD_RWLOCK_INITIALIZER */
-	(void) pthread_mutex_lock(&cache_lock);
-	if (w_wait > 0 || w_active > 0)
-	{
-		r_wait++;
-		while (w_wait > 0 || w_active > 0)
-			(void) pthread_cond_wait(&reader_go, &cache_lock);
-		r_wait--;
-	}
-	r_active++;
-	(void) pthread_mutex_unlock(&cache_lock);
-# endif /* PTHREAD_RWLOCK_INITIALIZER */
-#else /* DB_VERSION_CHECK(2,0,0) */
-	(void) pthread_mutex_lock(&cache_lock);
-#endif /* DB_VERSION_CHECK(2,0,0) */
-}
-
-/*
-**  DKIM_CACHE_READ_UNLOCK -- release a read lock
-**
-**  Parameters:
-**  	None.
-**
-**  Return value:
-**  	None.
-*/
-
-static void
-dkim_cache_read_unlock(void)
-{
-#if DB_VERSION_CHECK(2,0,0)
-# ifdef PTHREAD_RWLOCK_INITIALIZER
-	(void) pthread_rwlock_unlock(&cache_rwlock);
-# else /* PTHREAD_RWLOCK_INITIALIZER */
-	(void) pthread_mutex_lock(&cache_lock);
-	r_active--;
-	if (r_active == 0 && w_wait > 0)
-		(void) pthread_cond_signal(&writer_go);
-	(void) pthread_mutex_unlock(&cache_lock);
-# endif /* PTHREAD_RWLOCK_INITIALIZER */
-#else /* DB_VERSION_CHECK(2,0,0) */
-	(void) pthread_mutex_unlock(&cache_lock);
-#endif /* DB_VERSION_CHECK(2,0,0) */
-}
-
-/*
-**  DKIM_CACHE_WRITE_LOCK -- acquire a write lock
-**
-**  Parameters:
-**  	None.
-**
-**  Return value:
-**  	None.
-*/
-
-static void
-dkim_cache_write_lock(void)
-{
-#if DB_VERSION_CHECK(2,0,0)
-# ifdef PTHREAD_RWLOCK_INITIALIZER
-	(void) pthread_rwlock_wrlock(&cache_rwlock);
-# else /* PTHREAD_RWLOCK_INITIALIZER */
-	(void) pthread_mutex_lock(&cache_lock);
-	if (w_active > 0 || r_active > 0)
-	{
-		w_wait++;
-		while (w_active > 0 || r_active > 0)
-			(void) pthread_cond_wait(&writer_go, &cache_lock);
-		w_wait--;
-	}
-	w_active++;
-	assert(w_active == 1);
-	(void) pthread_mutex_unlock(&cache_lock);
-# endif /* PTHREAD_RWLOCK_INITIALIZER */
-#else /* DB_VERSION_CHECK(2,0,0) */
-	(void) pthread_mutex_lock(&cache_lock);
-#endif /* DB_VERSION_CHECK(2,0,0) */
-}
-
-/*
-**  DKIM_CACHE_WRITE_UNLOCK -- release a write lock
-**
-**  Parameters:
-**  	None.
-**
-**  Return value:
-**  	None.
-*/
-
-static void
-dkim_cache_write_unlock(void)
-{
-#if DB_VERSION_CHECK(2,0,0)
-# ifdef PTHREAD_RWLOCK_INITIALIZER
-	(void) pthread_rwlock_unlock(&cache_rwlock);
-# else /* PTHREAD_RWLOCK_INITIALIZER */
-	(void) pthread_mutex_lock(&cache_lock);
-	w_active--;
-	assert(w_active == 0);
-	if (w_wait > 0)
-		(void) pthread_cond_broadcast(&writer_go);
-	else if (r_wait > 0)
-		(void) pthread_cond_broadcast(&reader_go);
-	(void) pthread_mutex_unlock(&cache_lock);
-# endif /* PTHREAD_RWLOCK_INITIALIZER */
-#else /* DB_VERSION_CHECK(2,0,0) */
-	(void) pthread_mutex_unlock(&cache_lock);
-#endif /* DB_VERSION_CHECK(2,0,0) */
-}
 
 /*
 **  DKIM_CACHE_INIT -- initialize an on-disk cache of entries
@@ -222,53 +85,39 @@ dkim_cache_init(int *err, char *tmpdir)
 	c_expired = 0;
 
 	(void) pthread_mutex_init(&cache_stats_lock, NULL);
-
-#if DB_VERSION_MAJOR == 1
 	(void) pthread_mutex_init(&cache_lock, NULL);
-#else /* DB_VERSION_MAJOR == 1 */
-# ifdef PTHREAD_RWLOCK_INITIALIZER
-	(void) pthread_rwlock_init(&cache_rwlock, NULL);
-# else /* PTHREAD_RWLOCK_INITIALIZER */
-	(void) pthread_mutex_init(&cache_lock, NULL);
-	(void) pthread_cond_init(&reader_go, NULL);
-	(void) pthread_cond_init(&writer_go, NULL);
-
-	r_wait = 0;
-	r_active = 0;
-	w_wait = 0;
-# endif /* PTHREAD_RWLOCK_INITIALIZER */
-#endif /* DB_VERSION_MAJOR == 1 */
 
 #if DB_VERSION_CHECK(3,0,0)
 	status = db_create(&cache, NULL, 0);
 	if (status == 0)
 	{
-# if DB_VERSION_CHECK(4,0,0)
-#  if DB_VERSION_CHECK(4,2,0)
+# if DB_VERSION_CHECK(4,2,0)
+		/* tell libdb which temporary directory to use */
 		if (tmpdir != NULL && tmpdir[0] != '\0')
 		{
 			DB_ENV *env = NULL;
 
-#   if DB_VERSION_CHECK(4,3,0)
+#  if DB_VERSION_CHECK(4,3,0)
 			env = cache->get_env(cache);
-#   else /* DB_VERSION_CHECK(4,3,0) */
+#  else /* DB_VERSION_CHECK(4,3,0) */
 			(void) cache->get_env(cache, &env);
-#   endif /* DB_VERISON_CHECK(4,3,0) */
+#  endif /* DB_VERISON_CHECK(4,3,0) */
 
 			if (env != NULL)
 				(void) env->set_tmp_dir(env, tmpdir);
 		}
-#  endif /* DB_VERISON_CHECK(4,2,0) */
+# endif /* DB_VERISON_CHECK(4,2,0) */
 
+# if DB_VERSION_CHECK(4,1,25)
 		status = cache->open(cache, NULL, NULL, NULL, DB_HASH,
-		                     (DB_CREATE|DB_THREAD), DB_MODE);
-# else /* DB_VERSION_CHECK(4,0,0) */
+		                     DB_CREATE, DB_MODE);
+# else /* DB_VERSION_CHECK(4,1,25) */
 		status = cache->open(cache, NULL, NULL, DB_HASH,
-		                     (DB_CREATE|DB_THREAD), DB_MODE);
-# endif /* DB_VERSION_CHECK(4,0,0) */
+		                     DB_CREATE, DB_MODE);
+# endif /* DB_VERSION_CHECK(4,1,25) */
 	}
 #elif DB_VERSION_CHECK(2,0,0)
-	status = db_open(NULL, DB_HASH, (DB_CREATE|DB_THREAD), DB_MODE,
+	status = db_open(NULL, DB_HASH, DB_CREATE, DB_MODE,
 	                 NULL, NULL, &cache);
 #else /* ! DB_VERSION_CHECK(2,0,0) */
 	cache = dbopen(NULL, (O_CREAT|O_RDWR), DB_MODE, DB_HASH, NULL);
@@ -284,10 +133,6 @@ dkim_cache_init(int *err, char *tmpdir)
 		return NULL;
 	}
 
-#ifdef DEBUG
-	printf("libdkim cache initialized\n");
-#endif /* DEBUG */
-
 	return cache;
 }
 
@@ -332,10 +177,6 @@ dkim_cache_query(DB *db, char *str, int ttl, char *buf, size_t *buflen,
 	q.data = str;
 	q.size = strlen(q.data);
 
-#ifdef DEBUG
-	printf("libdkim cache query for `%s'\n", str);
-#endif /* DEBUG */
-
 #if DB_VERSION_CHECK(3,0,0)
 	d.flags = DB_DBT_USERMEM;
 	d.data = (void *) &ce;
@@ -348,7 +189,7 @@ dkim_cache_query(DB *db, char *str, int ttl, char *buf, size_t *buflen,
 	c_queries++;
 	pthread_mutex_unlock(&cache_stats_lock);
 
-	dkim_cache_read_lock();
+	pthread_mutex_lock(&cache_lock);
 
 #if DB_VERSION_CHECK(2,0,0)
 	status = db->get(db, NULL, &q, &d, 0);
@@ -356,7 +197,7 @@ dkim_cache_query(DB *db, char *str, int ttl, char *buf, size_t *buflen,
 	status = db->get(db, &q, &d, 0);
 #endif /* DB_VERSION_CHECK(2,0,0) */
 
-	dkim_cache_read_unlock();
+	pthread_mutex_unlock(&cache_lock);
 
 	if (status == 0)
 	{
@@ -368,10 +209,6 @@ dkim_cache_query(DB *db, char *str, int ttl, char *buf, size_t *buflen,
 			ce.cache_ttl = ttl;
 		if (ce.cache_when + ce.cache_ttl < now)
 		{
-#ifdef DEBUG
-			printf("libdkim cache query for `%s' expired\n", str);
-#endif /* DEBUG */
-
 			pthread_mutex_lock(&cache_stats_lock);
 			c_expired++;
 			pthread_mutex_unlock(&cache_stats_lock);
@@ -385,24 +222,15 @@ dkim_cache_query(DB *db, char *str, int ttl, char *buf, size_t *buflen,
 
 		sm_strlcpy(buf, ce.cache_data, *buflen);
 		*buflen = strlen(ce.cache_data);
-#ifdef DEBUG
-		printf("libdkim cache query for `%s' found: `%s'\n", str, buf);
-#endif /* DEBUG */
 		return 0;
 	}
 	else if (status != DB_NOTFOUND)
 	{
 		*err = status;
-#ifdef DEBUG
-		printf("libdkim cache query for `%s' error\n");
-#endif /* DEBUG */
 		return -1;
 	}
 	else
 	{
-#ifdef DEBUG
-		printf("libdkim cache query for `%s' not found\n", str);
-#endif /* DEBUG */
 		return 1;
 	}
 }
@@ -451,7 +279,7 @@ dkim_cache_insert(DB *db, char *str, char *data, int ttl, int *err)
 	ce.cache_ttl = ttl;
 	sm_strlcpy(ce.cache_data, data, sizeof ce.cache_data);
 
-	dkim_cache_write_lock();
+	pthread_mutex_lock(&cache_lock);
 
 #if DB_VERSION_CHECK(2,0,0)
 	status = db->put(db, NULL, &q, &d, 0);
@@ -459,21 +287,15 @@ dkim_cache_insert(DB *db, char *str, char *data, int ttl, int *err)
 	status = db->put(db, &q, &d, 0);
 #endif /* DB_VERSION_CHECK(2,0,0) */
 
-	dkim_cache_write_unlock();
+	pthread_mutex_unlock(&cache_lock);
 
 	if (status == 0)
 	{
-#ifdef DEBUG
-		printf("libdkim cache insert for `%s': `%s'\n", str, data);
-#endif /* DEBUG */
 		return 0;
 	}
 	else
 	{
 		*err = status;
-#ifdef DEBUG
-		printf("libdkim cache insert for `%s' error\n");
-#endif /* DEBUG */
 		return -1;
 	}
 }
@@ -518,14 +340,14 @@ dkim_cache_expire(DB *db, int ttl, int *err)
 
 	(void) time(&now);
 
-	dkim_cache_write_lock();
+	pthread_mutex_lock(&cache_lock);
 
 #if DB_VERSION_CHECK(2,0,0)
 	status = db->cursor(db, NULL, &dbc, 0);
 	if (status != 0)
 	{
 		*err = status;
-		dkim_cache_write_unlock();
+		pthread_mutex_unlock(&cache_lock);
 		return -1;
 	}
 #endif /* DB_VERSION_CHECK(2,0,0) */
@@ -610,7 +432,7 @@ dkim_cache_expire(DB *db, int ttl, int *err)
 	(void) dbc->c_close(dbc);
 #endif /* DB_VERSION_CHECK(2,0,0) */
 
-	dkim_cache_write_unlock();
+	pthread_mutex_unlock(&cache_lock);
 
 	return deleted;
 }
@@ -636,18 +458,7 @@ dkim_cache_close(DB *db)
 	(void) db->close(db);
 #endif /* DB_VERSION_CHECK(2,0,0) */
 
-#if DB_VERSION_MAJOR == 1
 	(void) pthread_mutex_destroy(&cache_lock);
-#else /* DB_VERSION_MAJOR == 1 */
-# ifdef PTHREAD_RWLOCK_INITIALIZER
-	(void) pthread_rwlock_destroy(&cache_rwlock);
-# else /* PTHREAD_RWLOCK_INITIALIZER */
-	(void) pthread_mutex_destroy(&cache_lock);
-	(void) pthread_cond_destroy(&reader_go);
-	(void) pthread_cond_destroy(&writer_go);
-# endif /* PTHREAD_RWLOCK_INITIALIZER */
-#endif /* DB_VERSION_MAJOR == 1 */
-
 }
 
 /*
