@@ -7,9 +7,12 @@
 #define _DKIM_CANON_H_
 
 #ifndef lint
-static char dkim_canon_h_id[] = "@(#)$Id: dkim-canon.h,v 1.4 2007/08/13 13:18:16 msk Exp $";
+static char dkim_canon_h_id[] = "@(#)$Id: dkim-canon.h,v 1.5 2007/10/07 13:04:02 msk Exp $";
 #endif /* !lint */
 
+/* system includes */
+#include <sys/types.h>
+
 /* libdkim includes */
 #include "dkim.h"
 
@@ -25,6 +28,7 @@ extern DKIM_STAT dkim_canon_closebody __P((DKIM *));
 extern void dkim_canon_free __P((DKIM *, DKIM_CANON *));
 extern DKIM_STAT dkim_canon_getfinal __P((DKIM_CANON *, u_char **, size_t *));
 extern DKIM_STAT dkim_canon_init __P((DKIM *, bool, bool));
+extern u_long dkim_canon_minbody __P((DKIM *));
 extern DKIM_STAT dkim_canon_runheaders __P((DKIM *, bool));
 extern DKIM_STAT dkim_canon_signature __P((DKIM *, struct dkim_header *));
 
