@@ -4,7 +4,7 @@
 */
 
 #ifndef lint
-static char dkim_tables_c_id[] = "@(#)$Id: dkim-tables.c,v 1.36 2008/03/13 13:03:06 msk Exp $";
+static char dkim_tables_c_id[] = "@(#)$Id: dkim-tables.c,v 1.38 2008/04/11 19:06:22 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -22,18 +22,18 @@ static char dkim_tables_c_id[] = "@(#)$Id: dkim-tables.c,v 1.36 2008/03/13 13:03
 #include "dkim.h"
 
 /* lookup tables */
-static struct chartable prv_keyparams[] =	/* key parameters */
+static struct nametable prv_keyparams[] =	/* key parameters */
 {
-	{ 'a',		DKIM_KEY_ALGORITHM },
-	{ 'g',		DKIM_KEY_GRANULARITY },
-	{ 'n',		DKIM_KEY_NOTES },
-	{ 'p',		DKIM_KEY_DATA },
-	{ 's',		DKIM_KEY_SERVICE },
-	{ 't',		DKIM_KEY_FLAGS },
-	{ 'v',		DKIM_KEY_VERSION },
-	{ '\0',		-1 }
+	{ "a",		DKIM_KEY_ALGORITHM },
+	{ "g",		DKIM_KEY_GRANULARITY },
+	{ "n",		DKIM_KEY_NOTES },
+	{ "p",		DKIM_KEY_DATA },
+	{ "s",		DKIM_KEY_SERVICE },
+	{ "t",		DKIM_KEY_FLAGS },
+	{ "v",		DKIM_KEY_VERSION },
+	{ NULL,		-1 }
 };
-struct chartable *keyparams = prv_keyparams;
+struct nametable *keyparams = prv_keyparams;
 
 static struct nametable prv_policyflags[] =	/* policy flags */
 {
@@ -207,6 +207,8 @@ static struct nametable prv_sigerrors[] =	/* signature parsing errors */
 	{ "signature verification failed",	DKIM_SIGERROR_BADSIG },
 	{ "unauthorized subdomain",		DKIM_SIGERROR_SUBDOMAIN },
 	{ "multiple keys found",		DKIM_SIGERROR_MULTIREPLY },
+	{ "header list tag empty",		DKIM_SIGERROR_EMPTY_H },
+	{ "header list missing required entries", DKIM_SIGERROR_INVALID_H },
 	{ NULL,					-1 },
 };
 struct nametable *sigerrors = prv_sigerrors;
@@ -268,61 +270,3 @@ dkim_name_to_code(struct nametable *tbl, const char *name)
 			return tbl[c].tbl_code;
 	}
 }
-
-/*
-**  DKIM_CODE_TO_CHAR -- translate a mnemonic code to its matching character
-**
-**  Parameters:
-**  	tbl -- name table
-**  	code -- code to translate
-**
-**  Return value:
-**  	Character representing the mnemonic code provided, or '\0' if
-**  	not found.
-*/
-
-const int
-dkim_code_to_char(struct chartable *tbl, const int code)
-{
-	int c;
-
-	assert(tbl != NULL);
-
-	for (c = 0; ; c++)
-	{
-		if (tbl[c].tbl_code == -1 && tbl[c].tbl_char == '\0')
-			return '\0';
-
-		if (tbl[c].tbl_code == code)
-			return tbl[c].tbl_char;
-	}
-}
-
-/*
-**  DKIM_CODE_TO_CHAR -- translate a mnemonic code to its matching character
-**
-**  Parameters:
-**  	tbl -- name table
-**  	chr -- character to translate
-**
-**  Return value:
-**  	Mnemonic representing the character code provided, or '\0' if
-**  	not found.
-*/
-
-const int
-dkim_char_to_code(struct chartable *tbl, const int chr)
-{
-	int c;
-
-	assert(tbl != NULL);
-
-	for (c = 0; ; c++)
-	{
-		if (tbl[c].tbl_code == -1 && tbl[c].tbl_char == '\0')
-			return -1;
-
-		if (tbl[c].tbl_char == chr)
-			return tbl[c].tbl_code;
-	}
-}
