@@ -1,5 +1,5 @@
 /*
-**  Copyright (c) 2005-2007 Sendmail, Inc. and its suppliers.
+**  Copyright (c) 2005-2008 Sendmail, Inc. and its suppliers.
 **    All rights reserved.
 */
 
@@ -7,7 +7,7 @@
 #define _DKIM_TYPES_H_
 
 #ifndef lint
-static char dkim_types_h_id[] = "@(#)$Id: dkim-types.h,v 1.83 2007/12/18 01:31:08 msk Exp $";
+static char dkim_types_h_id[] = "@(#)$Id: dkim-types.h,v 1.86 2008/02/26 00:04:31 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -45,7 +45,6 @@ struct dkim_pstate
 	unsigned int		ps_pflags;
 	int			ps_qstatus;
 	int			ps_state;
-	dkim_handling_t		ps_handling;
 	dkim_policy_t		ps_policy;
 };
 
@@ -187,6 +186,8 @@ struct dkim
 	bool			dkim_skipbody;
 	int			dkim_mode;
 	int			dkim_state;
+	int			dkim_chunkstate;
+	int			dkim_chunksm;
 	int			dkim_timeout;
 	int			dkim_presult;
 	int			dkim_hdrcnt;
