@@ -7,7 +7,7 @@
 #define _DKIM_H_
 
 #ifndef lint
-static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.171 2008/03/23 07:38:38 msk Exp $";
+static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.173 2008/04/11 19:06:22 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -31,7 +31,7 @@ static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.171 2008/03/23 07:38:38 msk Exp $
 **  	pp == patch number
 */
 
-#define	DKIM_LIB_VERSION	0x02050200
+#define	DKIM_LIB_VERSION	0x02050300
 
 /* the basics */
 #ifndef NULL
@@ -175,6 +175,8 @@ typedef int DKIM_SIGERROR;
 #define DKIM_SIGERROR_BADSIG		28	/* signature mismatch */
 #define DKIM_SIGERROR_SUBDOMAIN		29	/* unauthorized subdomain */
 #define DKIM_SIGERROR_MULTIREPLY	30	/* multiple records returned */
+#define DKIM_SIGERROR_EMPTY_H		31	/* h= empty */
+#define DKIM_SIGERROR_INVALID_H		32	/* h= missing req'd entries */
 
 /*
 **  DKIM_CANON -- canonicalization method
